/*    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package esrpatcher;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;

import util.Utils;

public class ESRPatch
{
    private static final int LBA_SIZE = 2048;

    private static final byte[] DVDVbuffer = Utils.getFileContent("resources/dump/DVDV.data");
    private static byte[] buffer = new byte[LBA_SIZE];
    private static byte[] emptyBuffer = new byte[LBA_SIZE];

    private static void patchLBABlock(RandomAccessFile iso, byte[] buf, int blockIndex) throws Exception
    {
        byte[] b;
        int desc_crc, desc_crc_len, tag_checksum;

        // Updates new referenced LBA: Now it points to a DVD_VIDEO structure
        buf[0xBC] = (byte) 0x80;
        buf[0xBD] = (byte) 0x00;

        // Calculates new checksums
        desc_crc_len = (((int) buf[11] & 0xFF) << 8 & 0xFF00) + ((int) buf[10] & 0xFF);
        b = new byte[desc_crc_len];
        for (int i = 0; i < desc_crc_len; i++)
        {
            b[i] = buf[i + 16];
        }

        desc_crc = CRC.CRC16CCITT(b);
        buf[8] = (byte) (desc_crc & 0xFF);
        buf[9] = (byte) ((desc_crc >> 8) & 0xFF);

        tag_checksum = 0;
        for (int i = 0; i < 16; i++)
        {
            if (i != 4)
            {
                tag_checksum += buf[i];
            }
        }

        buf[4] = (byte) tag_checksum;

        // Save modified LBA
        iso.seek(blockIndex * LBA_SIZE);
        iso.write(buf);
    }

    public static ESRStatus check(File aFile)
    {
        ESRStatus result;
        RandomAccessFile iso = null;

        try
        {
            iso = new RandomAccessFile(aFile, "r");

            if (!isUDFISO(iso)) // Checks if image is an UDF ISO
            {
                result = ESRStatus.ERROR_NOT_UDF_ISO;
            }
            else if (isPatched(iso)) // Checks if image is patched
            {
                result = ESRStatus.WARN_ALREADY_PATCHED;
            }
            else
            {
                result = ESRStatus.WARN_NOT_PATCHED;
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            result = ESRStatus.ERROR_PATCHING;
        }
        finally
        {
            try
            {
                if (iso != null)
                {
                    iso.close();
                }
            }
            catch (Exception e)
            {}
        }
        return result;
    }
    
    public static ESRStatus patch(File aFile)
    {
        ESRStatus result;
        RandomAccessFile iso = null;

        try
        {
            iso = new RandomAccessFile(aFile, "rw");

            if (!isUDFISO(iso)) // Checks if image is an UDF ISO
            {
                result = ESRStatus.ERROR_NOT_UDF_ISO;
            }
            else if (isPatched(iso)) // Checks if image is already patched
            {
                result = ESRStatus.WARN_ALREADY_PATCHED;
            }
            else
            {
                // Backups LBA 34 into 14
                iso.seek(34 * LBA_SIZE);
                iso.read(buffer);

                iso.seek(14 * LBA_SIZE);
                iso.write(buffer);

                patchLBABlock(iso, buffer, 34);

                // Backups LBA 50 into 15
                iso.seek(50 * LBA_SIZE);
                iso.read(buffer);

                iso.seek(15 * LBA_SIZE);
                iso.write(buffer);

                patchLBABlock(iso, buffer, 50);

                // Writes DVDV data
                iso.seek(128 * LBA_SIZE);
                iso.write(DVDVbuffer);

                result = ESRStatus.SUCCESS_PATCH;
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            result = ESRStatus.ERROR_PATCHING;
        }
        finally
        {
            try
            {
                if (iso != null)
                {
                    iso.close();
                }
            }
            catch (Exception e)
            {}
        }
        return result;
    }

    public static List<ESRStatus> patch(List<File> files)
    {
        List<ESRStatus> results = new ArrayList<ESRStatus>();
        for (File aFile : files)
        {
            if (aFile.exists() && aFile.isFile())
            {
                results.add(patch(aFile));
            }
            else
            {
                results.add(null);
            }
        }
        return results;
    }

    public static ESRStatus unPatch(File aFile)
    {
        ESRStatus result;
        RandomAccessFile iso = null;

        try
        {
            iso = new RandomAccessFile(aFile, "rw");

            if (!isUDFISO(iso)) // Checks if image is an UDF ISO
            {
                result = ESRStatus.ERROR_NOT_UDF_ISO;
            }
            else if (!isPatched(iso)) // Checks if image is patched
            {
                result = ESRStatus.WARN_NOT_PATCHED;
            }
            else
            {
                // Restore LBA 14 into 34
                iso.seek(14 * LBA_SIZE);
                iso.read(buffer);

                iso.seek(34 * LBA_SIZE);
                iso.write(buffer);

                // Restore LBA 15 into 50
                iso.seek(15 * LBA_SIZE);
                iso.read(buffer);

                iso.seek(50 * LBA_SIZE);
                iso.write(buffer);

                // Clear backups and dvdv data
                iso.seek(14 * LBA_SIZE);
                iso.write(emptyBuffer);

                iso.seek(15 * LBA_SIZE);
                iso.write(emptyBuffer);

                iso.seek(128 * LBA_SIZE);

                for (int i = 0; i < 12; i++)
                {
                    iso.write(emptyBuffer);
                }

                result = ESRStatus.SUCCES_UNPATCH;
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            result = ESRStatus.ERROR_PATCHING;
        }
        finally
        {
            try
            {
                if (iso != null)
                {
                    iso.close();
                }
            }
            catch (Exception e)
            {}
        }
        return result;
    }

    public static List<ESRStatus> unPatch(List<File> files)
    {
        List<ESRStatus> results = new ArrayList<ESRStatus>();
        for (File aFile : files)
        {
            if (aFile.exists() && aFile.isFile())
            {
                results.add(unPatch(aFile));
            }
            else
            {
                results.add(null);
            }
        }
        return results;
    }

    private static boolean isPatched(RandomAccessFile iso)
    {
        try
        {
            iso.seek(14 * LBA_SIZE);
            iso.read(buffer);

            // Checks for "+NSR" magic string
            if (buffer[25] == (byte) 0x2B && buffer[26] == (byte) 0x4E && buffer[27] == (byte) 0x53
                    && buffer[28] == (byte) 0x52)
            {
                return true;
            }
        }
        catch (Exception ex)
        {
            return false;
        }

        return false;
    }

    private static boolean isUDFISO(RandomAccessFile iso)
    {
        try
        {
            for (int i = 1; i < 64; i++)
            {
                iso.seek(LBA_SIZE * i + 32768);
                iso.read(buffer);

                // "NSR"
                if (buffer[1] == (byte) 0x4E && buffer[2] == (byte) 0x53 && buffer[3] == (byte) 0x52)
                    return true;
            }
        }
        catch (Exception ex)
        {
            return false;
        }

        return false;
    }

    /*private static int CRC16CCITT(byte[] buf)
    {
        int crc = 0;
        int polynomial = 0x1021;

        for (byte elem : buf)
        {
            for (int i = 0; i < 8; i++)
            {
                boolean bit = ((elem >> (7 - i) & 1) == 1);
                boolean c15 = ((crc >> 15 & 1) == 1);
                crc <<= 1;
                if (c15 ^ bit)
                {
                    crc ^= polynomial;
                }
            }
        }

        crc &= 0xffff;
        return crc;
    }*/
}
