package opl;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
import java.util.List;

import oplutil.device.ItemBase;

public class SupportBase
{
    public static final int BASE_GAME_NAME_MAX = 32;
    public static final int BASE_GAME_STARTUP_MAX = 12;

    private static final String extractString(byte[] buffer, int offset, int max)
    {
        String result = new String(buffer, offset, max);
        int end = result.indexOf(0x00);
        if (end != -1)
        {
            result = result.substring(0, end);
        }

        return result;
    }

    public static boolean sbIsSameSize(String prefix, int prevSize)
    {
        long size = -1;
        String path = prefix + "%sul.cfg";

        File fd = new File(path);
        if (fd.exists())
        {
            size = fd.length();
        }

        return size != prevSize;
    }

    private static int isValidIsoName(String name)
    {
        // SCUS_XXX.XX.ABCDEFGHIJKLMNOP.iso

        // Minimum is 17 char, GameID (11) + "." (1) + filename (1 min.) + ".iso" (4)
        int size = name.length();
        if ((size >= 17) && (name.charAt(4) == '_') && (name.charAt(8) == '.') && (name.charAt(11) == '.')
                && (name.endsWith(".iso")))
        {
            size -= 16;
            if (size <= BASE_GAME_NAME_MAX)
            {
                return size;
            }
        }

        return 0;
    }

    private static int scanForISO(String path, byte type, List<ItemBase> glist)
    {
        int size, count = 0;
        File fd = new File(path);

        File[] files = fd.listFiles();
        if (files != null)
        {
            for (File file : files)
            {
                if ((size = isValidIsoName(file.getName())) > 0)
                {
                    ItemBase game = new ItemBase();
                    game.name = file.getName().substring(BASE_GAME_STARTUP_MAX, BASE_GAME_STARTUP_MAX + size);
                    game.startup = file.getName().substring(0, BASE_GAME_STARTUP_MAX - 1);
                    game.parts = 0x01;
                    game.media = type;
                    game.isISO = true;

                    glist.add(game);
                    count++;
                }
            }
        }

        return count;
    }

    public static int sbReadList(List<ItemBase> list, String prefix, String sep)
    {
        File fd;
        int count = 0, size, fsize = -1;

        list.clear();

        // count iso games in "cd" directory
        count += scanForISO(prefix + "/CD", (byte) 0x12, list);

        // count iso games in "dvd" directory
        count += scanForISO(prefix + "/DVD", (byte) 0x14, list);

        // count and process games in ul.cfg
        fd = new File(prefix + "/ul.cfg");
        if (fd.exists())
        {
            byte buffer[] = new byte[0x040];

            size = (int) fd.length();
            fsize = size;
            count += size / 0x040;

            if (count > 0)
            {
                BufferedInputStream bis = null;
                try
                {
                    bis = new BufferedInputStream(new FileInputStream(fd));
                    while (size > 0)
                    {
                        bis.read(buffer, 0, 0x40);

                        ItemBase g = new ItemBase();

                        g.name = extractString(buffer, 0, BASE_GAME_NAME_MAX);
                        g.startup = extractString(buffer, BASE_GAME_NAME_MAX + 3, BASE_GAME_STARTUP_MAX);
                        g.parts = buffer[47];
                        g.media = buffer[48];
                        g.isISO = false;
                        list.add(g);

                        size -= 0x40;
                    }
                }
                catch (Exception ex)
                {
                    ex.printStackTrace(System.err);
                }
                finally
                {
                    if (bis != null)
                    {
                        try
                        {
                            bis.close();
                        }
                        catch (Exception ex)
                        {}
                    }
                }
            }
        }

        return fsize;
    }

    public static void sbRebuildULCfg(List<ItemBase> items, String prefix, ItemBase item)
    {
        // First do a backup
        File backup = new File(prefix + "/ul.bak");
        if (!backup.exists())
        {
            new File(prefix + "/ul.cfg").renameTo(backup);
        }
        
        RandomAccessFile file = null;
        try
        {
            file = new RandomAccessFile(prefix + "/ul.cfg", "rw");
            int pos = 0;
            for (ItemBase game : items)
            {
                if (!game.isISO && !game.equals(item))
                {
                    file.seek(pos);
                    file.write(game.name.getBytes(), 0, Math.min(BASE_GAME_NAME_MAX, game.name.length()));
                    file.seek(pos + BASE_GAME_NAME_MAX);
                    file.writeByte(0x75); // u
                    file.writeByte(0x6C); // l
                    file.writeByte(0x2E); // .
                    file.write(game.startup.getBytes(), 0, Math.min(BASE_GAME_STARTUP_MAX, game.startup.length()));
                    file.seek(pos + 47);
                    file.writeByte(game.parts);
                    file.writeByte(game.media);
                    file.seek(pos + 53);
                    file.writeByte(0x08); // just to be compatible with original ul.cfg
                    pos += 0x40;
                }
            }
            file.setLength(pos);
        }
        catch (Exception ex)
        {
            ex.printStackTrace(System.err);
        }
        finally
        {
            try
            {
                file.close();
            }
            catch (Exception ex)
            {
                ex.printStackTrace(System.err);
            }
        }
    }

    public static void sbDelete(List<ItemBase> items, String prefix, int id)
    {
        File path;
        ItemBase game = items.get(id);

        if (game.isISO)
        {
            if (game.media == 0x12)
            {
                path = new File(String.format("%s/CD/%s.%s.iso", prefix, game.startup, game.name));
            }
            else
            {
                path = new File(String.format("%s/DVD/%s.%s.iso", prefix, game.startup, game.name));
            }
            path.delete();
        }
        else
        {
            String crc = OplSystem.USBA_crc32(game.name);
            int i = 0;
            do
            {
                path = new File(String.format("%s/ul.%s.%s.%02x", prefix, crc, game.startup, Integer.valueOf(i)));
                i++;
                path.delete();
            } while (i < game.parts);

            sbRebuildULCfg(items, prefix, game);
        }
    }

    public static void sbRename(List<ItemBase> items, String prefix, int id, String newname)
    {
        File oldpath, newpath;
        ItemBase game = items.get(id);

        if (game.isISO)
        {
            if (game.media == 0x12)
            {
                oldpath = new File(String.format("%s/CD/%s.%s.iso", prefix, game.startup, game.name));
                newpath = new File(String.format("%s/CD/%s.%s.iso", prefix, game.startup, newname));
            }
            else
            {
                oldpath = new File(String.format("%s/DVD/%s.%s.iso", prefix, game.startup, game.name));
                newpath = new File(String.format("%s/DVD/%s.%s.iso", prefix, game.startup, newname));
            }
            oldpath.renameTo(newpath);
        }
        else
        {
            String pathStr = "%s/ul.%s.%s.%02x";
            String oldcrc = OplSystem.USBA_crc32(game.name);
            String newcrc = OplSystem.USBA_crc32(newname);
            int i = 0;
            do
            {
                oldpath = new File(String.format(pathStr, prefix, oldcrc, game.startup, Integer.valueOf(i)));
                newpath = new File(String.format(pathStr, prefix, newcrc, game.startup, Integer.valueOf(i)));
                i++;
                oldpath.renameTo(newpath);
            } while (i < game.parts);

            game.name = newname;
            sbRebuildULCfg(items, prefix, null);
        }
    }
}
