package oplutil.device;

import java.awt.Image;
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import opl.Config;
import opl.Lang;
import opl.OplSystem;
import opl.SupportBase;
import opl.Textures;
import oplutil.Elem;
import oplutil.Main;
import util.ISOSupport;
import util.Utils;
import esrpatcher.ESRPatch;
import esrpatcher.ESRStatus;

public class MenuBase
{
    class ULFileFilter implements FileFilter
    {
        private String basePath;
        private ItemBase item;

        public ULFileFilter(ItemBase itemToUpdate, String baseFileName)
        {
            basePath = baseFileName;
            item = itemToUpdate;
        }

        @Override
        public boolean accept(File pathname)
        {
            String filePath = pathname.getAbsolutePath();
            if (filePath.startsWith(basePath))
            {
                int lastDot = filePath.lastIndexOf('.');
                filePath = filePath.substring(lastDot + 1, filePath.length());

                if (filePath.length() == 2)
                {
                    try
                    {
                        Integer.parseInt(filePath);
                        item.size += pathname.length();
                        return true;
                    }
                    catch (Exception ex)
                    {}
                }
            }

            return false;
        }
    }

    protected Main main;

    protected int index;
    protected String prefKey;
    protected String prefValue;
    protected boolean isFAT32;
    
    public Elem elem;

    protected int curIndex = -1;
    protected int psIndex = -1;

    protected List<ItemBase> items;

    protected List<Elem> hints;

    public MenuBase(Main parent, int id, String key, boolean fat32, int textId, int iconId)
    {
        main = parent;
        index = id;
        prefKey = key;
        isFAT32 = fat32;
        elem = new Elem(textId, iconId);
        
        hints = initHints();
        items = new ArrayList<ItemBase>();
    }

    public int getIndex()
    {
        return index;
    }

    public boolean isFat32()
    {
        return isFAT32;
    }
    
    public List<Elem> getHints()
    {
        return hints;
    }

    public void refreshMenu(boolean force)
    {
        String newValue = main.getPrefs().getString(prefKey, null);
        
        if ((newValue == null && prefValue == null) || (prefValue != null && prefValue.equals(newValue)))
        {
            if (!force)
            {
                return;
            }
        }
        
        prefValue = newValue;
        populateItems(newValue);
        Collections.sort(items);
        reset();
    }

    protected void populateItems(String newValue)
    {
        if (newValue == null)
        {
            items.clear();
        }
        else
        {
            SupportBase.sbReadList(items, prefValue, "/");
            SupportBase.sbRebuildULCfg(items, prefValue, null);
        }
    }

    public int getItemsNumber()
    {
        return items.size();
    }

    public ItemBase getItem(int index)
    {
        return items.get(index);
    }

    public ItemBase getCurrentItem()
    {
        ItemBase result = null;

        if (curIndex != -1)
        {
            result = items.get(curIndex);
        }

        return result;
    }

    public int getCurrentIndex()
    {
        return curIndex;
    }

    public void setCurrentIndex(int index)
    {
        curIndex = index;
        if (main.isGamePanelDisplayed()) // TODO
        {
            inspectItem(getCurrentItem());
            main.getGamePanel().initItem(this, getCurrentItem());
        }
    }

    public int getPageStartIndex()
    {
        return psIndex;
    }

    public void setPageStartIndex(int index)
    {
        psIndex = index;
    }

    protected void reset()
    {
        if (!items.isEmpty())
        {
            curIndex = 0;
            psIndex = 0;
        }
        else
        {
            curIndex = -1;
            psIndex = -1;
        }
    }

    public Image getArt(String path)
    {
        return Textures.texDiscoverLoad(prefValue + "/ART/" + path, -1);
    }

    protected void fetchSub(String baseUrl, ItemBase currItem, StringBuffer currItemSB, String type)
    {
        String endPartJPG, endPartPNG;

        currItemSB.setLength(0);
        currItemSB.append(currItem.startup).append(type).append(".jpg");
        endPartJPG = currItemSB.toString();
        File destJPG = new File(prefValue + "/ART/" + endPartJPG);

        currItemSB.setLength(currItemSB.length() - 4);
        currItemSB.append(".png");
        endPartPNG = currItemSB.toString();
        File destPNG = new File(prefValue + "/ART/" + endPartPNG);
        
        if (!destJPG.exists() && !destPNG.exists())
        {
            if (!destJPG.exists())
            {
                Utils.downloadImage(baseUrl + "/art/" + endPartJPG, destJPG);
            }
            
            if (!destPNG.exists())
            {
                Utils.downloadImage(baseUrl + "/art/" + endPartPNG, destPNG);
            }
        }
    }

    public void fetchArt()
    {
        if (prefValue != null && prefValue.length() != 0)
        {
            new File(prefValue + "/ART").mkdir();

            String baseUrl = main.getPrefs().getString(Config.PREFS_DOWNLOAD_URL, null);
            if (baseUrl != null)
            {
                StringBuffer sb = new StringBuffer();
                for (ItemBase anItem : items)
                {
                    fetchSub(baseUrl, anItem, sb, ItemBase.ART_ICON);
                    fetchSub(baseUrl, anItem, sb, ItemBase.ART_COVER);
                    fetchSub(baseUrl, anItem, sb, ItemBase.ART_BACKGROUND);
                }
            }
        }
    }

    protected List<Elem> initHints()
    {
        List<Elem> result = new ArrayList<Elem>();

        result.add(new Elem(Lang._STR_SETTINGS, Textures.START_ICON));
        result.add(new Elem(Lang._STR_RUN, Textures.CROSS_ICON));
        result.add(new Elem(Lang._STR_COMPAT_SETTINGS, Textures.TRIANGLE_ICON));
        result.add(new Elem(Lang._STR_RENAME, Textures.CIRCLE_ICON));
        result.add(new Elem(Lang._STR_DELETE, Textures.SQUARE_ICON));

        return result;
    }

    public void changeESR(ItemBase item)
    {
        File target;
        if (item.isISO)
        {
            target = new File(item.fullName);
        }
        else
        {
            target = new File(item.fullName + ".00");
        }
        if (item.esrStatus == ESRStatus.WARN_ALREADY_PATCHED)
        {
            ESRPatch.unPatch(target);
        }
        else
        {
            ESRPatch.patch(target);
        }
        item.esrStatus = ESRPatch.check(target);
    }

    public void changeMediaType(ItemBase item)
    {
        if (item.isISO)
        {
            File oldpath, newpath;
            
            if (item.media == 0x12)
            {
                new File(prefValue + "/DVD").mkdir();
                oldpath = new File(String.format("%s/CD/%s.%s.iso", prefValue, item.startup, item.name));
                newpath = new File(String.format("%s/DVD/%s.%s.iso", prefValue, item.startup, item.name));
            }
            else
            {
                new File(prefValue + "/CD").mkdir();
                oldpath = new File(String.format("%s/DVD/%s.%s.iso", prefValue, item.startup, item.name));
                newpath = new File(String.format("%s/CD/%s.%s.iso", prefValue, item.startup, item.name));
            }
            oldpath.renameTo(newpath);
        }
        else
        {
            if (item.media == 0x12)
            {
                item.media = 0x14;
            }
            else
            {
                item.media = 0x12;
            }
            SupportBase.sbRebuildULCfg(items, prefValue, null);
        }
        refreshMenu(true);
    }

    public void inspectItem(ItemBase item)
    {
        StringBuffer sb = new StringBuffer();
        if (item.isISO)
        {
            if (item.media == 0x12)
            {
                sb.append("CD/");
            }
            else
            {
                sb.append("DVD/");
            }
            sb.append(item.startup).append('.').append(item.name).append(".iso");
            item.leadingName = sb.toString();
            item.fullName = prefValue + '/' + item.leadingName;

            File game = new File(item.fullName);
            if (game.exists())
            {
                if (game.length() != 0)
                {
                    item.isValid = true;
                    item.size = game.length();
                    item.esrStatus = ESRPatch.check(game);
                    item.realStartup = ISOSupport.getStartup(item.fullName);
                }
                item.realParts = 1;
            }
        }
        else
        {
            sb.append("ul.").append(OplSystem.USBA_crc32(item.name)).append('.').append(item.startup);
            item.leadingName = sb.toString();
            item.fullName = prefValue + '/' + item.leadingName;

            item.size = 0;
            File base = new File(prefValue);
            File[] parts = base.listFiles(new ULFileFilter(item, item.fullName));

            item.realParts = parts.length;
            if (item.realParts > 0)
            {
                if (item.size > 0)
                {
                    item.isValid = true;
                    item.esrStatus = ESRPatch.check(parts[0]);
                    item.realStartup = ISOSupport.getStartup(parts[0].getAbsolutePath());
                }
            }
        }
    }
}
