/*    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package util;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Window;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import javax.imageio.ImageIO;

public class Utils
{
    private static String fullLocation = null;
    private static String pathLocation = null;

    public static String projectName = "OPLUtil";
    public static String projectOwner = "izdubar";
    public static String projectVersion = "0.1";

    public static boolean fullTrace = false;

    static
    {
        initLocation();
        initProjectProperties();
    }

    private static void processException(Exception ex)
    {
        if (fullTrace)
        {
            ex.printStackTrace(System.err);
        }
        else
        {
            System.err.println(ex.getMessage());
        }
    }

    public static boolean downloadImage(String source, File destination)
    {
        boolean result = false;
        InputStream stream = null;

        try
        {
            URL urlObject = new URL(source);
            URLConnection con = urlObject.openConnection();
            con.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)");
            stream = con.getInputStream();

            int len;
            byte[] buffer = new byte[4096];
            BufferedInputStream reader = new BufferedInputStream(stream);
            BufferedOutputStream writer = new BufferedOutputStream(new FileOutputStream(destination));

            while ((len = reader.read(buffer, 0, buffer.length)) != -1)
            {
                writer.write(buffer, 0, len);
            }

            writer.close();
            reader.close();
            result = true;
        }
        catch (Exception ex)
        {
            processException(ex);
        }
        finally
        {
            if (stream != null)
            {
                try
                {
                    stream.close();
                }
                catch (Exception ex)
                {
                    ex.printStackTrace();
                }
            }
        }

        return result;
    }

    public static Image getImage(String source)
    {
        return getImage(source, false);
    }

    public static Image getImage(String source, boolean canFail)
    {
        Image result = null;
        InputStream stream = null;

        try
        {
            stream = getResource(source);
            result = ImageIO.read(stream);
        }
        catch (Exception ex)
        {
            if (!canFail)
            {
                processException(ex);
            }
        }
        finally
        {
            if (stream != null)
            {
                try
                {
                    stream.close();
                }
                catch (Exception ex)
                {
                    ex.printStackTrace(System.err);
                }
            }
        }
        return result;
    }

    public static byte[] getFileContent(String source)
    {
        byte[] result = null;
        InputStream stream = null;

        try
        {
            stream = getResource(source);
            int len = stream.available();
            result = new byte[len];
            stream.read(result);
        }
        catch (Exception ex)
        {
            processException(ex);
        }
        finally
        {
            if (stream != null)
            {
                try
                {
                    stream.close();
                }
                catch (Exception ex)
                {
                    processException(ex);
                }
            }
        }
        return result;
    }

    public static InputStream getResource(String source) throws Exception
    {
        InputStream stream = null;
        File sourceFile = new File(source);
        if ((fullLocation != null) && !sourceFile.isAbsolute())
        {
            stream = new BufferedInputStream(Utils.class.getClassLoader().getResourceAsStream(source));
        }
        else
        {
            stream = new BufferedInputStream(new FileInputStream(sourceFile));
        }
        return stream;
    }

    public static void initProjectProperties()
    {
        Attributes currentAttributes = null;
        String currentAttribute = null;
        Manifest currentManifest = null;
        InputStream stream = null;
        URL currentURL;

        if (fullLocation == null)
        {
            return;
        }
        
        try
        {
            Enumeration<URL> myEnum = Utils.class.getClassLoader().getResources("META-INF/MANIFEST.MF");
            while (myEnum.hasMoreElements())
            {
                currentURL = myEnum.nextElement();
                if (currentURL.getFile().contains(fullLocation))
                {
                    stream = currentURL.openStream();
                    currentManifest = new Manifest(stream);
                    stream.close();
                    currentAttributes = currentManifest.getMainAttributes();

                    currentAttribute = currentAttributes.getValue("Implementation-Title");
                    if (currentAttribute != null)
                    {
                        projectName = currentAttribute;
                    }
                    currentAttribute = currentAttributes.getValue("Implementation-Vendor");
                    if (currentAttribute != null)
                    {
                        projectOwner = currentAttribute;
                    }
                    currentAttribute = currentAttributes.getValue("Implementation-Version");
                    if (currentAttribute != null)
                    {
                        projectVersion = currentAttribute;
                    }
                }
            }
        }
        catch (Exception ex)
        {
            processException(ex);
        }
        finally
        {
            try
            {
                if (stream != null)
                {
                    stream.close();
                }
            }
            catch (Exception ex)
            {
                processException(ex);
            }
        }
    }

    public static void initLocation()
    {
        try
        {
            String theName = new StringBuffer("/").append(Utils.class.getName().replace('.', '/')).append(".class")
                    .toString();
            String resource = Utils.class.getResource(theName).getFile();

            if (resource.startsWith("file"))
            {
                resource = resource.substring(6, resource.length());
            }

            int hasJar = resource.indexOf('!');
            if (hasJar != -1)
            {
                fullLocation = resource.substring(0, hasJar);
                int indJar = fullLocation.lastIndexOf('/');
                pathLocation = fullLocation.substring(0, indJar);
            }
            else
            {
                int indPath = resource.indexOf(theName);
                pathLocation = resource.substring(0, indPath);
            }
        }
        catch (Exception ex)
        {
            processException(ex);
        }
    }

    public static Dimension getDefaultSize()
    {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        screenSize.width = (int) (screenSize.width * 0.60);
        screenSize.height = (int) (screenSize.height * 0.60);
        return screenSize;
    }

    public static void centerResizeWindow(Window window, boolean resize)
    {
        int width = 0;
        int height = 0;

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        if (resize)
        {
            width = (int) (screenSize.width * 0.60);
            height = (int) (screenSize.height * 0.60);
            window.setBounds(Math.max(0, (screenSize.width - width) / 2),
                    Math.max(0, (screenSize.height - height) / 2), width, height);
        }
        else
        {
            Dimension winSize = window.getSize();
            window.setLocation(Math.max(0, (screenSize.width - winSize.width) / 2), Math.max(0,
                    (screenSize.height - winSize.height) / 2));
        }

    }

    //////////////////////////////////////// For graphic use  ////////////////////////////////////////

    public static List<File> convertURIListToFileList(String data)
    {
        List<File> list = new ArrayList<File>();

        StringTokenizer st = new StringTokenizer(data);
        while (st.hasMoreTokens())
        {
            String s = st.nextToken();
            if (!s.startsWith("#"))
            {
                try
                {
                    URI uri = new URI(s);
                    File file = new File(uri);
                    list.add(file);
                }
                catch (URISyntaxException ex)
                {
                    processException(ex);
                }
                catch (IllegalArgumentException ex)
                {
                    processException(ex);
                }
            }
        }
        return list;
    }

    public static String getLocation()
    {
        return pathLocation;
    }

    //////////////////////////////////////// For command line ////////////////////////////////////////

    public static List<File> scanInput(String[] args)
    {
        List<File> result = new ArrayList<File>();

        for (int i = 1; i < args.length; i++)
        {
            result.add(new File(args[i]));
        }
        return result;
    }
}
